# Generated from java-escape by ANTLR 4.5
from antlr4 import *

# This class defines a complete listener for a parse tree produced by PLambdaParser.
class PLambdaListener(ParseTreeListener):

    # Enter a parse tree produced by PLambdaParser#unit.
    def enterUnit(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#unit.
    def exitUnit(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#seqExpression.
    def enterSeqExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#seqExpression.
    def exitSeqExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#letExpression.
    def enterLetExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#letExpression.
    def exitLetExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#defineExpression.
    def enterDefineExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#defineExpression.
    def exitDefineExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#lambdaExpression.
    def enterLambdaExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#lambdaExpression.
    def exitLambdaExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#invokeExpression.
    def enterInvokeExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#invokeExpression.
    def exitInvokeExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#applyExpression.
    def enterApplyExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#applyExpression.
    def exitApplyExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#dataExpression.
    def enterDataExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#dataExpression.
    def exitDataExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#unaryExpression.
    def enterUnaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#unaryExpression.
    def exitUnaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#binaryExpression.
    def enterBinaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#binaryExpression.
    def exitBinaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#ternaryExpression.
    def enterTernaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#ternaryExpression.
    def exitTernaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#oneOrMoreExpression.
    def enterOneOrMoreExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#oneOrMoreExpression.
    def exitOneOrMoreExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#twoOrMoreExpression.
    def enterTwoOrMoreExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#twoOrMoreExpression.
    def exitTwoOrMoreExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#naryExpression.
    def enterNaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#naryExpression.
    def exitNaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#tryExpression.
    def enterTryExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#tryExpression.
    def exitTryExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#forExpression.
    def enterForExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#forExpression.
    def exitForExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#stringLiteral.
    def enterStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#stringLiteral.
    def exitStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#identifierLiteral.
    def enterIdentifierLiteral(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#identifierLiteral.
    def exitIdentifierLiteral(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#noneLiteral.
    def enterNoneLiteral(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#noneLiteral.
    def exitNoneLiteral(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#parameterList.
    def enterParameterList(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#parameterList.
    def exitParameterList(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#parameter.
    def enterParameter(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#parameter.
    def exitParameter(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#bindingList.
    def enterBindingList(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#bindingList.
    def exitBindingList(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#bindingPair.
    def enterBindingPair(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#bindingPair.
    def exitBindingPair(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#catchExpression.
    def enterCatchExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#catchExpression.
    def exitCatchExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#rangeExpression.
    def enterRangeExpression(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#rangeExpression.
    def exitRangeExpression(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#data.
    def enterData(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#data.
    def exitData(self, ctx):
        pass


    # Enter a parse tree produced by PLambdaParser#token.
    def enterToken(self, ctx):
        pass

    # Exit a parse tree produced by PLambdaParser#token.
    def exitToken(self, ctx):
        pass


